part of '../app_route.dart';

/// Activity route builder
typedef ActivityRouteBuilder = Route Function(RouteSettings settings);

/// [AppRoute] of an Activity
class ActivityRoute<T> extends AppRoute<T> {
  static final Map<String, ActivityRouteBuilder> _routes = {};
  final ActivityRouteBuilder builder;

  ActivityRoute(String name, {required this.builder}) : super(name) {
    _routes[name] = builder;
  }

  ActivityRoute.builder(
    Object name, {
    required RouteWidgetBuilder builder,
    ThemeBuilder? theme,
    SystemUiStyleBuilder systemUiStyle = Application.defaultSystemUiStyle,
    bool maintainState = true,
    bool fullscreenDialog = false,
  }) : this(name.toString(),
            builder: (settings) => MaterialPageRoute(
                builder: wrapIfNotNull<WidgetBuilder, ThemeBuilder>(
                    item: theme,
                    wrap: (child, theme) =>
                        (context) => Theme(data: theme(context, settings), child: Builder(builder: child)),
                    child: (context) => AnnotatedSystemUIStyle(
                        style: systemUiStyle(context), child: builder(context, settings.arguments as RouteArgs))),
                settings: settings,
                maintainState: maintainState,
                fullscreenDialog: fullscreenDialog));

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) {
    final raw = Navigator.of(context).pushNamed(name, arguments: args);
    return AppRoute._result(raw);
  }

  @override
  Future<T> openReplace(BuildContext context, {RouteArgs args = const {}, Object? result}) {
    final raw = Navigator.of(context).pushReplacementNamed(name, arguments: args, result: result);
    return AppRoute._result(raw);
  }

  static Route _onGenerateRoute(RouteSettings settings) {
    if (settings.arguments == null) settings = settings.copyWith(arguments: const <String, dynamic>{});
    final route = _routes[settings.name]?.call(settings);
    if (route == null) throw StateError('Route ${settings.name} not found');
    return route;
  }
}
